package main

import (
	"archive/zip"
	"bytes"
	"errors"
	"flag"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"strconv"
	"time"

	"github.com/antchfx/xmlquery"
	_ "github.com/influxdata/influxdb1-client"
	client "github.com/influxdata/influxdb1-client/v2"
	"gopkg.in/ini.v1"
)

// ParseArgs is not yet implemented
func ParseArgs() error {
	return nil
}

// GetConfig fetch config from ini file
func GetConfig(configfile string, fuelpricesconfig *FuelPricesConfig) error {
	flag.Usage = Usage

	config, err := ini.Load(configfile)
	if err != nil {
		return err
	}

	var fpc FuelPricesConfig

	fuelpricesSection := config.Section("fuelprices")
	fpc.RemoteURL = fuelpricesSection.Key("remote_url").MustString("https://donnees.roulez-eco.fr/opendata/instantane")
	fpc.RemoteFilename = fuelpricesSection.Key("remote_filename").MustString("PrixCarburants_instantane.xml")
	fpc.XPathBase = fuelpricesSection.Key("xpath_base").MustString(".//pdv[@id='%s']/prix[@nom='%s']")
	fpc.Table = fuelpricesSection.Key("table").MustString("fuel_price")
	fpc.Pos = fuelpricesSection.Key("pos").Strings(",")
	if len(fpc.Pos) < 1 {
		err := errors.New("No pos defined")
		return err
	}
	fpc.Types = fuelpricesSection.Key("types").Strings(",")
	if len(fpc.Types) < 1 {
		err := errors.New("No fuel types defined")
		return err
	}

	influxdbSection := config.Section("influxdb")
	fpc.InfluxHost = influxdbSection.Key("hostname").MustString("localhost")
	fpc.InfluxPort = influxdbSection.Key("port").MustInt(8086)
	fpc.InfluxUser = influxdbSection.Key("username").MustString("username")
	fpc.InfluxPass = influxdbSection.Key("password").MustString("password")
	fpc.InfluxDB = influxdbSection.Key("database").MustString("me")

	*fuelpricesconfig = fpc

	if err != nil {
		return err
	}

	return nil
}

// DownloadFile fetch file from webserver
func DownloadFile(fpc *FuelPricesConfig, szip *Srcfile) error {
	pollTo := 30 * time.Millisecond

	c := &http.Client{Timeout: pollTo * time.Second, Transport: &http.Transport{
		IdleConnTimeout:    pollTo,
		DisableCompression: false,
	}}
	//resp, err := http.Get(url)
	resp, err := c.Get(fpc.RemoteURL)
	if err != nil {
		return err
	}

	defer resp.Body.Close()

	szip.Content, err = ioutil.ReadAll(resp.Body)
	if err != nil {
		return err
	}

	time.Sleep(pollTo)

	return nil
}

// ExtractZip get the XML file to be processed
func ExtractZip(fpc *FuelPricesConfig, szip *Srcfile, output *[]byte) error {

	zipfile, err := zip.NewReader(bytes.NewReader(szip.Content), int64(len(szip.Content)))
	if err != nil {
		return err
	}

	for _, f := range zipfile.File {
		if f.Name == fpc.RemoteFilename {
			rc, err := f.Open()
			if err != nil {
				return err
			}
			*output, err = ioutil.ReadAll(rc)
			rc.Close()
		} else {
			log.Fatal("File not found")
		}
	}
	return err
}

// GetPrices parses the XML file and get values of prices
func GetPrices(prices **[]Price, fpc *FuelPricesConfig, output *[]byte) error {

	var pr []Price
	var xml *xmlquery.Node

	f := bytes.NewReader(*output)
	xml, err := xmlquery.Parse(f)
	if err != nil {
		return err
	}

	for _, station := range fpc.Pos {
		for _, fuel := range fpc.Types {

			query := fmt.Sprintf(fpc.XPathBase, station, fuel)
			list := xmlquery.FindOne(xml, query)

			if list != nil {
				for _, i := range list.Attr {
					if i.Name.Local == "valeur" {
						var val float64
						if s, err := strconv.ParseFloat(i.Value, 64); err == nil {
							val = s
						}
						pr = append(pr, Price{ID: station, Fuel: fuel, Amount: val})
					}
				}
			} else {
				log.Println(fmt.Sprintf("Fuel type not found for point of sale, skipping. Query : %s", query))
			}
		}
	}
	*prices = &pr
	return err
}

// SendToInflux sends time series data to influxdb
func SendToInflux(fpc *FuelPricesConfig, prices *[]Price) error {
	httpClient, err := client.NewHTTPClient(client.HTTPConfig{
		Addr:     fmt.Sprintf("http://%s:%d", fpc.InfluxHost, fpc.InfluxPort),
		Username: fpc.InfluxUser,
		Password: fpc.InfluxPass,
	})
	if err != nil {
		return err
	}

	bp, err := client.NewBatchPoints(client.BatchPointsConfig{
		Database: fpc.InfluxDB,
	})
	if err != nil {
		return err
	}

	for _, p := range *prices {

		tags := map[string]string{"pdv": p.ID, "fuel": p.Fuel}
		fields := map[string]interface{}{"value": p.Amount}

		point, _ := client.NewPoint(
			fpc.Table,
			tags,
			fields,
			time.Now(),
		)

		log.Println(point)

		bp.AddPoint(point)
		err = httpClient.Write(bp)
		if err != nil {
			return err
		}
	}

	return nil
}

// Usage displays possible arguments
func Usage() {
	flag.PrintDefaults()
	os.Exit(1)
}
